package HTML::FormEngine::SkinConfirm;

use strict;
use vars qw(@ISA $VERSION);
use HTML::FormEngine::Skin;
@ISA = qw(HTML::FormEngine::Skin);
$VERSION = '0.1';

sub _get_confirm_skin {
  return undef;
}
